#!/bin/bash

# $Id:$ */

#
# QosCosGrid Notification
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

#using IP address force UnixODBC to use INET sockets
DEFAULT_HOST="127.0.0.1"
DEFAULT_PORT="5432"
DEFAULT_DATABASE="qcg-ntf"
DEFAULT_USER="qcg-ntf"
DEFAULT_PASSWORD=`openssl rand -base64 24 || exit 3`
DEFAULT_CREATE_USER="y"
DEFAULT_CREATE_DATABASE="y"
DEFAULT_USE_LOCAL_PSQL="y"
SYSTEM_USER="qcg-ntf"
SCHEMA_FILE="/usr/share/qcg-ntf/tools/qcg-ntf-psql.sql"
ODBC_DATA_SOURCE="qcg-ntf"

# rollback data
USER_CREATED=
DATABASE_CREATED=
ODBC_CREATED=

#DEBUG=1

check_privileges() {
	if [ `id -u` -ne 0 ]; then
		echo Program must be run with root privileges
		exit 1
	fi
}

banner() {
cat << EOF
Welcome to qcg-ntf installation script!

This script will guide you through process of configuring proper environment
for running the QCG Notification service. You have to answer few questions regarding
parameters of your database. If you are not sure just press Enter and use the
default values.

EOF
}

run_cmd() {
	if [ -n "$DEBUG" ]; then
		for arg in "$@"; do
			echo -n "\"$arg\" "
		done
		echo
	fi
	if [ -n "$DEBUG" ]; then
		"$@"
	else
		"$@" >/dev/null 2>&1
	fi
}

check_postgresql_installed() {
	run_cmd rpm -q --quiet postgresql-server
}

install_postgresql() {
	run_cmd yum -y -q install postgresql-server
	run_cmd rpm -q --quiet postgresql-server
	run_cmd /sbin/chkconfig postgresql on
  if [[ -n `rpm -q --queryformat "%{RELEASE}\n" postgresql | grep -v el5` ]]; then
		run_cmd /sbin/service postgresql initdb || :
	fi
}

postgresql_running() {
	run_cmd /sbin/service postgresql status
}

start_postgresql() {
	run_cmd /sbin/chkconfig postgresql on
	run_cmd /sbin/service postgresql start
}

read_var() {
	read -p "$1 [$2]: " var
	if [ -z $var ]; then
		eval $3=$2
	else
		eval $3=$var
	fi
}

read_yn_var() {
	read_var "$1 (y/n)" $2 var

	while [ "$var" != "y" -a "$var" != "n" ]; do
		read_var "$1 (y/n)" $2 var
	done

	eval $3=$var
}

read_config() {
	read_yn_var "Use local PostgreSQL server?" $DEFAULT_USE_LOCAL_PSQL USE_LOCAL_PSQL

	if [ "x$USE_LOCAL_PSQL" != "xy" ]; then
		read_var "Host" $DEFAULT_HOST HOST
		read_var "Port" $DEFAULT_PORT PORT
	else
		HOST=$DEFAULT_HOST
		PORT=$DEFAULT_PORT
	fi

	read_var "Database" $DEFAULT_DATABASE DATABASE
	read_var "User" $DEFAULT_USER USER
	read_var "Password" $DEFAULT_PASSWORD PASSWORD

	if [ "x$USE_LOCAL_PSQL" == "xy" ]; then
		read_yn_var "Create database?" $DEFAULT_CREATE_DATABASE CREATE_DATABASE
		read_yn_var "Create user?" $DEFAULT_CREATE_USER CREATE_USER
	fi
}

check_system_user() {
	run_cmd id $SYSTEM_USER
}

local_psql() {
	user=$1
	shift
	run_cmd su - $user -c "psql $@"
}

remote_psql() {                                                                                                                                                                                                                                                 
        user=$1                                                                                                                                                                                                                                                 
        shift                                                                                                                                                                                                                                                   
        run_cmd su - $user -c "psql -h \"$HOST\" -p $PORT -U \"$USER\" $@"                                                                                                                                                                              
}                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                
create_user() {                                                                                                                                                                                                                                                 
        local_psql postgres "-c \"CREATE ROLE \\\"$USER\\\" PASSWORD '$PASSWORD' LOGIN;\""                                                                                                                                                                      
}                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                
delete_user() {                                                                                                                                                                                                                                                 
        local_psql postgres "-c \"DROP ROLE \\\"$USER;\\\""                                                                                                                                                                                                     
}                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                
create_database() {                                                                                                                                                                                                                                             
        local_psql postgres "-c \"CREATE DATABASE \\\"$DATABASE\\\" OWNER \\\"$USER\\\";\""                                                                                                                                                                     
}                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                
delete_database() {                                                                                                                                                                                                                                             
        local_psql postgres "-c \"DROP DATABASE \\\"$DATABASE\\\";\""                                                                                                                                                                                           
}   

create_schema() {
	$1 $SYSTEM_USER "-d $DATABASE -f $SCHEMA_FILE"
}

check_odbc_data_source() {
	run_cmd odbcinst -q -s -n $1 > /dev/null 2>&1
}

create_odbc_data_source() {
	ODBC_SOURCE="[$ODBC_DATA_SOURCE]
Description=qcg-ntf
Driver=/usr/lib64/psqlodbc.so
Database=$DATABASE
Servername=$HOST
Port=$PORT
ReadOnly=No"
	echo "$ODBC_SOURCE"  >> /etc/odbc.ini
}

delete_odbc_data_source() {
	ODBC_SOURCE_LEN=`echo "$ODBC_SOURCE" | wc -l`
	ODBC_SOURCE_LEN=$(( $ODBC_SOURCE_LEN - 1 ))
	TMPNAME=`mktemp`
	grep -q -v -A $ODBC_SOURCE_LEN "\\[$ODBC_DATA_SOURCE\\]" /etc/odbc.ini > $TMPNAME
	mv $TMPNAME /etc/odbc.ini
}

print_var() {
	eval echo $1 = \\\"\$$1\\\"
}

status() {
	if [ $? -eq 0 ]; then
		echo OK
	else
		echo Failed
		if [ -z "$ROLLBACK" ]; then
			rollback
		fi
		exit 1
	fi
}

rollback() {
	echo "Rolling back changes..."
	ROLLBACK=1
	if [ -n "$ODBC_CREATED" ]; then
		echo -n " * Delete ODBC data source..."
		delete_odbc_data_source
		status
	fi

	if [ -n "$DATABASE_CREATED" ]; then
		echo -n " * Delete database $DATABASE..."
		delete_database
		status
	fi

	if [ -n "$USER_CREATED" ]; then
		echo -n " * Delete user $USER..."
		delete_user
		status
	fi
}

# start

check_privileges
banner
read_config

if [ -n "$DEBUG" ]; then
	for var in USE_LOCAL_PSQL CREATE_USER CREATE_DATABASE HOST PORT DATABASE USER PASSWORD; do
		print_var $var
	done
fi

echo
echo -n "Checking for system user $SYSTEM_USER..."
check_system_user
status

if [ "$USE_LOCAL_PSQL" == "y" ]; then
	echo -n "Checking whether PostgreSQL server is installed..."
	check_postgresql_installed
	if [ $? -ne 0 ]; then
		echo
		read_yn_var "PostgreSQL server is not installed. Install now?" y INSTALL_POSTGRES

		if [ $INSTALL_POSTGRES == "y" ]; then
			echo -n "Installing PostgreSQL server..."
			install_postgresql
			status
		fi
	else
		echo OK
	fi

	echo -n "Checking whether PostgreSQL server is running..."
	postgresql_running
	if [ $? -ne 0 ]; then
		echo
		read_yn_var "PostgreSQL server is not running. Start now?" y START_POSTGRES

		if [ $START_POSTGRES == "y" ]; then
			echo  -n "Starting PostgreSQL server..."
			start_postgresql
			sleep 5
			status
		fi
	else
		echo OK
	fi
fi

echo
echo "Performing installation"


if [ $USE_LOCAL_PSQL == "y" ]; then
	PSQL=local_psql
	if [ $CREATE_USER == "y" ]; then
		echo -n " * Creating user $USER..."
		create_user
		status
		USER_CREATED=y
	fi

	if [ $CREATE_DATABASE == "y" ]; then
		echo -n " * Creating database $DATABASE..."
		create_database
		status
		DATABASE_CREATED=y
	fi
else
	PSQL=remote_psql
fi

echo -n " * Creating database schema..."
create_schema $PSQL
status

echo -n " * Checking for ODBC data source $ODBC_DATA_SOURCE..."
check_odbc_data_source qcg-ntf

if [ $? -eq 0 ]; then
	echo OK
else
	echo 
	echo -n " * Installing ODBC data source..."
	create_odbc_data_source
	status
	ODBC_CREATED=y
fi

echo
echo "The newly established database settings must be reflected in the Database section of the QCG-Notification configuration file" 
echo "(by default /etc/qcg/qcg-ntf/qcg-ntfd.xml)"
echo 
echo "Remember to add appropriate entry to /var/lib/pgsql/data/pg_hba.conf (as the first rule!) to allow user $USER to"
echo "access database $DATABASE. For instance:"
echo
echo -e "host\t$DATABASE\t$USER\t127.0.0.1/32\tmd5"
echo
echo "and reload Postgres server."
echo
