-- $Id: qcg-ntf-psql.sql 3 2010-11-26 17:07:13Z bartek $
--
-- QosCosGrid Notification
-- Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--
-- This file should be used to set up database tables for Notification Service.

DROP TABLE current_notifications CASCADE;
DROP TABLE publishers_registrations CASCADE;
DROP TABLE subscriptions CASCADE;
DROP TABLE pullpoints CASCADE;
DROP TABLE pullpoints_notifications_buffer CASCADE;
DROP TABLE notifications_buffer CASCADE;
DROP TABLE undelivered_notifications CASCADE;

DROP SEQUENCE pullpoints_notifications_buffer_seq CASCADE;

CREATE SEQUENCE pullpoints_notifications_buffer_seq START WITH 1 INCREMENT BY 1;

CREATE TABLE current_notifications
(
    keyval VARCHAR(256),
    CONSTRAINT keyval_pk PRIMARY KEY(keyval),
    msg VARCHAR (4096),
    publisherAddr VARCHAR (256),
    timestamp TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC')
);

CREATE TABLE publishers_registrations
(
	ID VARCHAR(256),
	CONSTRAINT ID_pub_pk PRIMARY KEY(ID),
	publisherAddr VARCHAR(256),
	creationTime TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
	terminationTime TIMESTAMP
);

CREATE TABLE subscriptions
(
	ID VARCHAR(256),
	CONSTRAINT ID_sub_pk PRIMARY KEY(ID),
	consumerAddr VARCHAR(256),
	subscriberAddr VARCHAR(256),
	filterExprElement VARCHAR(1024),
	creationTime TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
	terminationTime TIMESTAMP,
	xsltDocument VARCHAR(4096)
);

CREATE TABLE pullpoints
(
	ID VARCHAR(256) NOT NULL,
	CONSTRAINT ID_pull_pk PRIMARY KEY(ID),
	creatorAddr VARCHAR(256),
	creationTime TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
	terminationTime TIMESTAMP,
	modificationTime TIMESTAMP,
	maxSize INTEGER,
	actualSize INTEGER
);

CREATE TABLE pullpoints_notifications_buffer
(
	num INTEGER NOT NULL DEFAULT nextval('pullpoints_notifications_buffer_seq'),
	CONSTRAINT ID_pullnotbuf_pk PRIMARY KEY(num),
	pullpointID VARCHAR(256) NOT NULL,
	FOREIGN KEY (pullpointID) REFERENCES pullpoints (ID) ON DELETE CASCADE,
	tns VARCHAR(256),
	tnsPrefix VARCHAR(256),
	topic VARCHAR(256),
	dialect VARCHAR(256),
    msg VARCHAR (4096),
	publisherAddr VARCHAR (256),
    timestamp TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC')
);

CREATE TABLE notifications_buffer
(
	id VARCHAR(36),
	CONSTRAINT ID_resendnotbuf_pk PRIMARY KEY(id),
	tns VARCHAR(256),
	tnsPrefix VARCHAR(256),
	topic VARCHAR(256),
	dialect VARCHAR(256),
    msg VARCHAR (4096),
	publisherAddr VARCHAR (256),
    timestamp TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
    referenceCounter INTEGER
);

CREATE TABLE undelivered_notifications
(
	subscriptionID VARCHAR(256) NOT NULL,
	FOREIGN KEY (subscriptionID) REFERENCES subscriptions (ID) ON DELETE CASCADE,
	notificationID VARCHAR(36) NOT NULL,
	FOREIGN KEY (notificationID) REFERENCES notifications_buffer (ID) ON DELETE CASCADE,
	CONSTRAINT ID_undeliverednot_pk PRIMARY KEY(subscriptionID, notificationID),
	timestamp TIMESTAMP DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC')	
)
