#!/bin/sh

# $Id:$ */

#
# QosCosGrid Notification
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

#using IP address force UnixODBC to use INET sockets
DEFAULT_HOST="127.0.0.1"
DEFAULT_PORT="5432"
DEFAULT_DATABASE="qcg-ntf"
DEFAULT_USER="qcg-ntf"
DEFAULT_PASSWORD="qcg-ntf"
DEFAULT_USE_LOCAL_PSQL="y"

SYSTEM_USER="qcg-ntf"
SCHEMA_FILE="/usr/share/qcg-ntf/tools/qcg-ntf-psql.sql"

# rollback data
USER_CREATED=
DATABASE_CREATED=
ODBC_CREATED=

#DEBUG=1

check_privileges() {
	if [ `id -u` -ne 0 ]; then
		echo Program must be run with root privileges
		exit 1
	fi
}

banner() {
cat << EOF
Welcome to qcg-ntf update script!

This script will guide you through process of updating QCG Notification service. 
You have to answer few questions regarding parameters of your database. 
If you are not sure check the values in QCG-Notification configuration file.

EOF
}

run_cmd() {
	if [ -n "$DEBUG" ]; then
		for arg in "$@"; do
			echo -n "\"$arg\" "
		done
		echo
	fi
	if [ -n "$DEBUG" ]; then
		"$@"
	else
		"$@" >/dev/null 2>&1
	fi
}

read_var() {
	read -p "$1 [$2]: " var
	if [ -z $var ]; then
		eval $3=$2
	else
		eval $3=$var
	fi
}

read_yn_var() {
	read_var "$1 (y/n)" $2 var

	while [ "$var" != "y" -a "$var" != "n" ]; do
		read_var "$1 (y/n)" $2 var
	done

	eval $3=$var
}

read_config() {
	read_yn_var "Use local PostgreSQL server?" $DEFAULT_USE_LOCAL_PSQL USE_LOCAL_PSQL

	if [ "x$USE_LOCAL_PSQL" != "xy" ]; then
		read_var "Host" $DEFAULT_HOST HOST
		read_var "Port" $DEFAULT_PORT PORT
	else
		HOST=$DEFAULT_HOST
		PORT=$DEFAULT_PORT
	fi

	read_var "Database" $DEFAULT_DATABASE DATABASE
	read_var "User" $DEFAULT_USER USER
	read_var "Password" $DEFAULT_PASSWORD PASSWORD
}

check_system_user() {
	run_cmd id $SYSTEM_USER
}

local_psql() {
	user=$1
	shift
	run_cmd su - $user -c "psql -h\"$HOST\" -p \"$PORT\" @"
}

remote_psql() {                                                                                                                                                                                                                                                 
        user=$1                                                                                                                                                                                                                                                 
        shift                                                                                                                                                                                                                                                   
        run_cmd su - $user -c "psql -h \"$HOST\" -p \"$PORT\" -U \"$USER\" $@"                                                                                                                                                                              
}

create_schema() {
	$1 $SYSTEM_USER "-d $DATABASE -f $SCHEMA_FILE"
}

print_var() {
	eval echo $1 = \\\"\$$1\\\"
}

status() {
	if [ $? -eq 0 ]; then
		echo OK
	else
		echo Failed
		exit 1
	fi
}

# start

check_privileges
banner
read_config

if [ -n "$DEBUG" ]; then
	for var in USE_LOCAL_PSQL HOST PORT DATABASE USER PASSWORD; do
		print_var $var
	done
fi

echo
echo -n "Checking for system user $SYSTEM_USER..."
check_system_user
status

echo
echo "Performing installation"


if [ $USE_LOCAL_PSQL == "y" ]; then
	PSQL=local_psql
else
	PSQL=remote_psql
fi

echo
echo "Schema update"

echo -n " * Creating database schema..."
create_schema $PSQL
status

echo
